require 'roxml'
require 'nokogiri'
require 'mdata/types'

module Salesforce
	module Metadata
		# A Salesforce PermissionSet metadata object.
		#
		# See https://developer.salesforce.com/docs/atlas.en-us.api_meta.meta/api_meta/meta_permissionset.htm
		#
		# @author Ben Burwell
		class PermissionSet
		  include ROXML

		  attr_accessor :filename
		  
		  xml_name 'PermissionSet'


			xml_accessor :applicationVisibilities,
				as: [Salesforce::Types::PermissionSetApplicationVisibility],
				from: 'applicationVisibilities'
			
			xml_accessor :classAccesses,
				as: [Salesforce::Types::PermissionSetApexClassAccess],
				from: 'classAccesses'

			xml_accessor :customPermissions,
				as: [Salesforce::Types::PermissionSetCustomPermissions],
				from: 'customPermissions'

			xml_accessor :description

			xml_accessor :externalDataSourceAccesses,
				as: [Salesforce::Types::PermissionSetExternalDataSourceAccess],
				from: 'externalDataSourceAccesses'

			xml_accessor :fieldPermissions,
				as: [Salesforce::Types::PermissionSetFieldPermissions],
				from: 'fieldPermissions'

			xml_accessor :label

			xml_accessor :objectPermissions,
				as: [Salesforce::Types::PermissionSetObjectPermissions],
				from: 'objectPermissions'

			xml_accessor :pageAccesses,
				as: [Salesforce::Types::PermissionSetApexPageAccess],
				from: 'pageAccesses'

			xml_accessor :recordTypeVisibilities,
				as: [Salesforce::Types::PermissionSetRecordTypeVisibility],
				from: 'recordTypeVisibilities'

			xml_accessor :tabSettings,
				as: [Salesforce::Types::PermissionSetTabSetting],
				from: 'tabSettings'

			xml_accessor :userLicense

			xml_accessor :userPermissions,
				as: [Salesforce::Types::PermissionSetUserPermission],
				from: 'userPermissions'

		  # Read a permissionset file from its name and directory
		  #
		  # The default file path is `./src/permissionsets/NAME.permissionset`, but the
		  # directory can be overridden by passing a non-nil String.
		  #
		  # @param name [String] the name of the permissionset to read, like 'Admin'
		  # @param dir [String] the directory to search for the permissionset in. If
		  #   nil, defaults to `./src/permissionsets`.
		  # @return [PermissionSet] the permissionset object that has been instantiated
		  def self.read name, dir
		  	dir = './src/permissionsets' if dir.nil?
		  	filename = "#{dir}/#{name}.permissionset"
		  	permissionset = PermissionSet.from_xml(File.read(filename))
		  	permissionset.filename = filename
		  	permissionset
		  end

		  # Save the permissionset to disk, writing it to the file from whence it came
		  # 
		  # Relies on the permissionset object having a `@filename` to save to.
		  def save
		  	doc = ::Nokogiri::XML::Document.new
		  	doc.root = to_xml()
		  	doc.root.add_namespace nil, 'http://soap.sforce.com/2006/04/metadata'
		  	File.open @filename, 'w' do |file|
					file << doc.to_xml(:indent => 4, :encoding => 'UTF-8')
		  	end
		  end

		  # Create an empty permissionset by name in directory
		  #
		  # @param name [String] the name of the permissionset to create
		  # @param dir [String] the directory to create it in, defaulting to
		  #   `./src/permissionsets`.
		  def self.touch name, dir
		  	dir = './src/permissionsets' if dir.nil?
		  	filename = "#{dir}/#{name}.permissionset"
		  	permissionset = PermissionSet.new
		  	permissionset.filename = filename
		  	permissionset.save
		  end
		end
	end
end
