require 'roxml'
require 'nokogiri'
require 'mdata/types'

module Salesforce
	module Metadata
		# A Salesforce Profile metadata object.
		#
		# See https://developer.salesforce.com/docs/atlas.en-us.api_meta.meta/api_meta/meta_profile.htm
		#
		# @author Ben Burwell
		class Profile
		  include ROXML

		  attr_accessor :filename
		  
		  xml_name 'Profile'

		  xml_accessor :applicationVisibilities,
		  	as: [Salesforce::Types::ProfileApplicationVisibility],
		  	from: 'applicationVisibilities'
		  xml_accessor :classAccesses,
		  	as: [Salesforce::Types::ProfileApexClassAccess],
		  	from: 'classAccesses'
		  xml_accessor :custom
		  xml_accessor :customPermissions,
		  	as: [Salesforce::Types::ProfileCustomPermissions],
		  	from: 'customPermissions'
		  xml_accessor :description
		  xml_accessor :externalDataSourceAccesses,
		  	as: [Salesforce::Types::ProfileExternalDataSourceAccess],
		  	from: 'externalDataSourceAccesses'
		  xml_accessor :fieldLevelSecurities,
		  	as: [Salesforce::Types::ProfileFieldLevelSecurity],
		  	from: 'fieldLevelSecurities'
		  xml_accessor :fieldPermissions,
		  	as: [Salesforce::Types::ProfileFieldLevelSecurity],
		  	from: 'fieldPermissions'
		  xml_accessor :fullName
		  xml_accessor :layoutAssignments,
		  	as: [Salesforce::Types::ProfileLayoutAssignments],
		  	from: 'layoutAssignments'
		  xml_accessor :loginHours,
		  	as: [Salesforce::Types::ProfileLoginHours],
		  	from: 'loginHours'
		  xml_accessor :loginIpRanges,
		  	as: [Salesforce::Types::ProfileLoginIpRange],
		  	from: 'loginIpRanges'
		  xml_accessor :objectPermissions,
		  	as: [Salesforce::Types::ProfileObjectPermissions],
		  	from: 'objectPermissions'
		  xml_accessor :pageAccesses,
		  	as: [Salesforce::Types::ProfileApexPageAccess],
		  	from: 'pageAccesses'
		  xml_accessor :recordTypeVisibilities,
		  	as: [Salesforce::Types::ProfileRecordTypeVisibility],
		  	from: 'recordTypeVisibilities'
		  xml_accessor :tabVisibilities,
		  	as: [Salesforce::Types::ProfileTabVisibility],
		  	from: 'tabVisibilities'
		  xml_accessor :userLicense
		  xml_accessor :userPermissions,
		  	as: [Salesforce::Types::ProfileUserPermission],
		  	from: 'userPermissions'

		  # Read a profile file from its name and directory
		  #
		  # The default file path is `./src/profiles/NAME.profile`, but the
		  # directory can be overridden by passing a non-nil String.
		  #
		  # @param name [String] the name of the profile to read, like 'Admin'
		  # @param dir [String] the directory to search for the profile in. If
		  #   nil, defaults to `./src/profiles`.
		  # @return [Profile] the profile object that has been instantiated
		  def self.read name, dir
		  	dir = './src/profiles' if dir.nil?
		  	filename = "#{dir}/#{name}.profile"
		  	profile = Profile.from_xml(File.read(filename))
		  	profile.filename = filename
		  	profile
		  end

		  # Save the profile to disk, writing it to the file from whence it came
		  # 
		  # Relies on the profile object having a `@filename` to save to.
		  def save
		  	doc = ::Nokogiri::XML::Document.new
		  	doc.root = to_xml()
		  	doc.root.add_namespace nil, 'http://soap.sforce.com/2006/04/metadata'
		  	File.open @filename, 'w' do |file|
					file << doc.to_xml(:indent => 4, :encoding => 'UTF-8')
		  	end
		  end

		  # Create an empty profile by name in directory
		  #
		  # @param name [String] the name of the profile to create
		  # @param dir [String] the directory to create it in, defaulting to
		  #   `./src/profiles`.
		  def self.touch name, dir
		  	dir = './src/profiles' if dir.nil?
		  	filename = "#{dir}/#{name}.profile"
		  	profile = Profile.new
		  	profile.filename = filename
		  	profile.save
		  end
		end
	end
end
