require 'roxml'

module Salesforce
	module Types
		# A generic FieldLevelSecurity class that can be extended for specific
		# metadata objects, e.g. ProfileFieldLevelSecurity.
		#
		# @author Ben Burwell
		class FieldLevelSecurity
			include ROXML

			xml_accessor :editable
			xml_accessor :field
			xml_accessor :hidden
			xml_accessor :readable

			# Get all the permissions granted as an easy-to-use Array
			#
			# @author Ben Burwell
			# @return [Array] an array of strings representing the permissions that
			#   are granted.
			def get_permissions
				permissions = Array.new
				permissions.push 'Editable' if @editable == 'true'
				permissions.push 'Hidden' if @hidden == 'true'
				permissions.push 'Readable' if @readable == 'true'
				permissions
			end

			# Get a string representation of the permissions
			#
			# @author Ben Burwell
			# @return [String] the permissions granted on the field, comma separated
			def to_s
				get_permissions.join(', ')
			end

			# Produces a string representation of the permissions suitable for
			# displaying in a list, à la `ls -la`. The resulting list might look
			# something like this:
			#
			#     Readable Editable
			#                       Hidden
			#     Readable
			# 
			# @author Ben Burwell
			# @return [String] the permissions granted on the field, flag-style
			def to_flag_style
				flags = ''
				if @readable == 'true'
					flags += 'Readable '
				else
					flags += '         '
				end

				if @editable == 'true'
					flags += 'Editable '
				else
					flags += '         '
				end

				if @hidden == 'true'
					flags += 'Hidden'
				else
					flags += '      '
				end

				flags
			end

		end

		# See https://developer.salesforce.com/docs/atlas.en-us.api_meta.meta/api_meta/meta_profile.htm#profilefieldlevelsecurity_title
		class ProfileFieldLevelSecurity < FieldLevelSecurity; end
	end
end
